<?php
/**
 * @package     HSChromeLogger!
 * @author      Hosting Skills a.s.b.l. - http://www.hosting-skills.lu
 * @copyright   Copyright (C) 2014-2021 Hosting Skills a.s.b.l.. All rights reserved.
 * @license     GNU/GPLv3 https://www.gnu.org/licenses/gpl-3.0.html
 */

defined('_JEXEC') or die;

use Joomla\CMS\Installer\InstallerAdapter;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;

/**
 * HSChromeLogger! System Plugin Installer Script
 *
 * @since   1.0.0
 */
class plgSystemHSChromeLoggerInstallerScript {
	/**
	 * Minimum Joomla! Version for the Component.
	 *
	 * @const  string
	 *
	 * @since  1.6.0
	 */
	const joomla = '3.10.0';

	/**
	 * Minimum php Version for the Component.
	 *
	 * @var  string
	 *
	 * @since  1.6.0
	 */
	const php = '7.4.0';

	/**
	 * Function called before extension installation/update/removal procedure commences
	 *
	 * @param   string            $type    The type of change (install, update, discover_install or uninstall)
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.0.0
	 */
	public function preflight(string $type, InstallerAdapter $parent): bool
	{
		if ('uninstall' === $type)
		{
			return true;
		}

		// Check the minimum php version.
		if (version_compare(PHP_VERSION, self::php, 'lt'))
		{
			$parent->getParent()->message = '';
			Log::add(sprintf(Text::_('PLG_SYSTEM_HSCHROMELOGGER_PHP_VERSION_ERROR'), self::php), Log::WARNING, 'jerror');

			return false;
		}

		// Check the minimum Joomla! version.
		if (version_compare(JVERSION, self::joomla, 'lt'))
		{
			$parent->getParent()->message = '';
			Log::add(sprintf(Text::_('PLG_SYSTEM_HSCHROMELOGGER_JOOMLA_VERSION_ERROR'), self::joomla), Log::WARNING, 'error');

			return false;
		}

		if ('update' === $type)
		{
			$db    = $parent->getParent()->getDbo();
			$query = $db->getQuery(true)
				->select($db->quoteName('manifest_cache'))
				->from($db->quoteName('#__extensions'))
				->where($db->quoteName('name') . ' = ' . $db->quote($parent->getManifest()->name));
			$db->setQuery($query);

			$version  = $parent->getManifest()->version;

			try
			{
				$db->execute();
				$manifest       = json_decode($db->loadResult(), true);
				$update_version = $manifest['version'];
			}
			catch (RuntimeException $e)
			{
				$update_version = $version;
			}

			if (version_compare($version, $update_version, 'le'))
			{
				$parent->getParent()->message = '';
				Log::add(sprintf(Text::_('PLG_SYSTEM_HSCHROMELOGGER_UPDATE_VERSION_ERROR'), $update_version, $version), Log::WARNING, 'error');

				return false;
			}
		}

		return true;
	}

	/**
	 * This method is called after a component is installed.
	 *
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.0.0
	 */
	public function install(InstallerAdapter $parent): bool
	{
		$parent->getParent()->message = sprintf(Text::_('PLG_SYSTEM_HSCHROMELOGGER_INSTALL'), $parent->getManifest()->version) . '<br /><br /><span style="color: red;">' . Text::_('PLG_SYSTEM_HSCHROMELOGGER_CONFIGURE') . '</span><br /><br /><img src="../plugins/system/hschromelogger/hschromelogger.png" alt="HSChromeLogger!" /><br /><br />';

		return true;
	}

	/**
	 * This method is called after a component is updated.
	 *
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.0.0
	 */
	public function update(InstallerAdapter $parent): bool
	{
		$db    = $parent->getParent()->getDbo();
		$query = $db->getQuery(true)
			->select($db->quoteName('enabled'))
			->from($db->quoteName('#__extensions'))
			->where($db->quoteName('name') . ' = ' . $db->quote($parent->getManifest()->name));
		$db->setQuery($query);

		try
		{
			$db->execute();
			$enabled = $db->loadResult();
		}
		catch (RuntimeException $e)
		{
			$enabled = false;
		}

		$version  = $parent->getManifest()->version;

		// Fix update servers.
		if (version_compare($version, '1.3.1', 'lt'))
		{
			$query = $db->getQuery(true)
				->delete($db->quoteName('#__update_sites'))
				->where($db->quoteName('location') . ' = ' . $db->quote('http://www.hosting-skills.lu/updates/hschromelogger.xml'));
			$db->setQuery($query);
			$db->execute();
		}

		$parent->getParent()->message = sprintf(Text::_('PLG_SYSTEM_HSCHROMELOGGER_UPDATE'), $version) . ($enabled ? '' : '<br /><br /><span style="color: red;">' . Text::_('PLG_SYSTEM_HSCHROMELOGGER_CONFIGURE')) . '</span><br /><br /><img src="../plugins/system/hschromelogger/hschromelogger.png" alt="HSChromeLogger!" /><br /><br />';

		return true;
	}

	/**
	 * This method is called after a component is uninstalled.
	 *
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.3.1
	 */
	public function uninstall(InstallerAdapter $parent): bool
	{
		return true;
	}

	/**
	 * Function called after extension installation/update/removal procedure finishes
	 *
	 * @param   string            $type    The type of change (install, update, discover_install or uninstall)
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.3.1
	 */
	public function postflight(string $type, InstallerAdapter $parent): bool
	{
		return true;
	}
}